@@ -1336,3 +1336,27 @@ ALTER TABLE ps_endpoints ADD COLUMN stir_shaken_profile VARCHAR(80);
 
 UPDATE alembic_version SET version_num='a062185f355c' WHERE alembic_version.version_num = '7197536bb68d';
 
+-- Running upgrade a062185f355c -> 539f68bede2c
+
+ALTER TABLE ps_endpoints MODIFY `100rel` ENUM('no','required','peer_supported','yes') NULL;
+
+UPDATE alembic_version SET version_num='539f68bede2c' WHERE alembic_version.version_num = 'a062185f355c';
+
+-- Running upgrade 539f68bede2c -> 417c0247fd7e
+
+ALTER TABLE ps_endpoints ADD COLUMN security_negotiation ENUM('no','mediasec');
+
+ALTER TABLE ps_endpoints ADD COLUMN security_mechanisms VARCHAR(512);
+
+ALTER TABLE ps_registrations ADD COLUMN security_negotiation ENUM('no','mediasec');
+
+ALTER TABLE ps_registrations ADD COLUMN security_mechanisms VARCHAR(512);
+
+UPDATE alembic_version SET version_num='417c0247fd7e' WHERE alembic_version.version_num = '539f68bede2c';
+
+-- Running upgrade 417c0247fd7e -> ccf795ee535f
+
+ALTER TABLE ps_globals ADD COLUMN all_codecs_on_empty_reinvite ENUM('0','1','off','on','false','true','no','yes');
+
+UPDATE alembic_version SET version_num='ccf795ee535f' WHERE alembic_version.version_num = '417c0247fd7e';
+
