@@ -121,6 +121,7 @@
 							<enum name="ast_channel_dtmf_begin_type" />
 							<enum name="ast_channel_dtmf_end_type" />
 							<enum name="ast_channel_flash_type" />
+							<enum name="ast_channel_wink_type" />
 							<enum name="ast_channel_hold_type" />
 							<enum name="ast_channel_unhold_type" />
 							<enum name="ast_channel_chanspy_start_type" />
@@ -2297,7 +2298,7 @@ int stasis_message_type_declined(const char *name)
 	ao2_cleanup(name_in_declined);
 	ao2_ref(cfg, -1);
 	if (res) {
-		ast_log(LOG_NOTICE, "Declining to allocate Stasis message type '%s' due to configuration\n", name);
+		ast_debug(4, "Declining to allocate Stasis message type '%s' due to configuration\n", name);
 	}
 	return res;
 }
