@@ -33,6 +33,7 @@
 #include "asterisk/channel.h"
 #include "asterisk/module.h"
 #include "asterisk/callerid.h"
+#include "asterisk/conversions.h"
 
 /*!
  * \internal
@@ -44,13 +45,11 @@ static void set_id_from_hdr(pjsip_fromto_hdr *hdr, struct ast_party_id *id)
 {
 	char cid_name[AST_CHANNEL_NAME];
 	char cid_num[AST_CHANNEL_NAME];
-	pjsip_sip_uri *uri;
 	pjsip_name_addr *id_name_addr = (pjsip_name_addr *) hdr->uri;
 	char *semi;
 
-	uri = pjsip_uri_get_uri(id_name_addr);
 	ast_copy_pj_str(cid_name, &id_name_addr->display, sizeof(cid_name));
-	ast_copy_pj_str(cid_num, &uri->user, sizeof(cid_num));
+	ast_copy_pj_str(cid_num, ast_sip_pjsip_uri_get_username(hdr->uri), sizeof(cid_num));
 
 	/* Always truncate caller-id number at a semicolon. */
 	semi = strchr(cid_num, ';');
@@ -119,6 +118,47 @@ static pjsip_fromto_hdr *get_id_header(pjsip_rx_data *rdata, const pj_str_t *hea
 	return parsed_hdr;
 }
 
+/*!
+ * \internal
+ * \brief Set an ANI2 integer based on OLI data in a From header
+ *
+ * This uses the contents of a From header in order to set Originating Line information.
+ *
+ * \param rdata The incoming message
+ * \param ani2 The ANI2 field to set
+ * \retval 0 Successfully parsed OLI
+ * \retval non-zero Could not parse OLI
+ */
+static int set_id_from_oli(pjsip_rx_data *rdata, int *ani2)
+{
+	char oli[AST_CHANNEL_NAME];
+
+	pjsip_param *oli1, *oli2, *oli3;
+
+	static const pj_str_t oli_str1 = { "isup-oli", 8 };
+	static const pj_str_t oli_str2 = { "ss7-oli", 7 };
+	static const pj_str_t oli_str3 = { "oli", 3 };
+
+	pjsip_fromto_hdr *from = pjsip_msg_find_hdr(rdata->msg_info.msg,
+			PJSIP_H_FROM, rdata->msg_info.msg->hdr.next);
+
+	if (!from) {
+		return -1; /* This had better not happen */
+	}
+
+	if ((oli1 = pjsip_param_find(&from->other_param, &oli_str1))) {
+		ast_copy_pj_str(oli, &oli1->value, sizeof(oli));
+	} else if ((oli2 = pjsip_param_find(&from->other_param, &oli_str2))) {
+		ast_copy_pj_str(oli, &oli2->value, sizeof(oli));
+	} else if ((oli3 = pjsip_param_find(&from->other_param, &oli_str3))) {
+		ast_copy_pj_str(oli, &oli3->value, sizeof(oli));
+	} else {
+		return -1;
+	}
+
+	return ast_str_to_int(oli, ani2);
+}
+
 /*!
  * \internal
  * \brief Set an ast_party_id structure based on data in a P-Asserted-Identity header
@@ -342,7 +382,8 @@ static void update_incoming_connected_line(struct ast_sip_session *session, pjsi
 {
 	struct ast_party_id id;
 
-	if (!session->endpoint->id.trust_inbound) {
+	if (!session->endpoint->trust_connected_line
+		|| !session->endpoint->id.trust_inbound) {
 		return;
 	}
 
@@ -370,6 +411,7 @@ static void update_incoming_connected_line(struct ast_sip_session *session, pjsi
 static int caller_id_incoming_request(struct ast_sip_session *session, pjsip_rx_data *rdata)
 {
 	if (!session->channel) {
+		int ani2;
 		/*
 		 * Since we have no channel this must be the initial inbound
 		 * INVITE.  Set the session ID directly because the channel
@@ -386,6 +428,11 @@ static int caller_id_incoming_request(struct ast_sip_session *session, pjsip_rx_
 		if (!session->endpoint->id.self.number.valid) {
 			set_id_from_from(rdata, &session->id);
 		}
+		if (!set_id_from_oli(rdata, &ani2)) {
+			session->ani2 = ani2;
+		} else {
+			session->ani2 = 0;
+		}
 	} else {
 		/*
 		 * ReINVITE or UPDATE.  Check for changes to the ID and queue
@@ -418,6 +465,7 @@ static void caller_id_incoming_response(struct ast_sip_session *session, pjsip_r
  * \internal
  * \brief Create an identity header for an outgoing message
  * \param hdr_name The name of the header to create
+ * \param base
  * \param tdata The message to place the header on
  * \param id The identification information for the new header
  * \return newly-created header
@@ -430,8 +478,7 @@ static pjsip_fromto_hdr *create_new_id_hdr(const pj_str_t *hdr_name, pjsip_fromt
 
 	id_hdr = pjsip_from_hdr_create(tdata->pool);
 	id_hdr->type = PJSIP_H_OTHER;
-	pj_strdup(tdata->pool, &id_hdr->name, hdr_name);
-	id_hdr->sname = id_hdr->name;
+	id_hdr->sname = id_hdr->name = *hdr_name;
 
 	id_name_addr = pjsip_uri_clone(tdata->pool, base->uri);
 	id_uri = pjsip_uri_get_uri(id_name_addr->uri);
@@ -495,6 +542,7 @@ static void add_privacy_header(pjsip_tx_data *tdata, const struct ast_party_id *
 /*!
  * \internal
  * \brief Add a P-Asserted-Identity header to an outbound message
+ * \param session The session on which communication is happening
  * \param tdata The message to add the header to
  * \param id The identification information used to populate the header
  */
@@ -543,6 +591,33 @@ static void add_pai_header(const struct ast_sip_session *session, pjsip_tx_data
 	pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr *)pai_hdr);
 }
 
+/*!
+ * \internal
+ * \brief Add party parameter to a Remote-Party-ID header.
+ *
+ * \param tdata The message where the Remote-Party-ID header is
+ * \param hdr The header on which the parameters are being added
+ * \param session The session involved
+ */
+static void add_party_param(pjsip_tx_data *tdata, pjsip_fromto_hdr *hdr, const struct ast_sip_session *session)
+{
+	static const pj_str_t party_str = { "party", 5 };
+	static const pj_str_t calling_str = { "calling", 7 };
+	static const pj_str_t called_str = { "called", 6 };
+	pjsip_param *party;
+
+	/* The party value can't change throughout the lifetime, so it is set only once */
+	party = pjsip_param_find(&hdr->other_param, &party_str);
+	if (party) {
+		return;
+	}
+
+	party = PJ_POOL_ALLOC_T(tdata->pool, pjsip_param);
+	party->name = party_str;
+	party->value = (session->inv_session->role == PJSIP_ROLE_UAC) ? calling_str : called_str;
+	pj_list_insert_before(&hdr->other_param, party);
+}
+
 /*!
  * \internal
  * \brief Add privacy and screen parameters to a Remote-Party-ID header.
@@ -604,6 +679,7 @@ static void add_privacy_params(pjsip_tx_data *tdata, pjsip_fromto_hdr *hdr, cons
 /*!
  * \internal
  * \brief Add a Remote-Party-ID header to an outbound message
+ * \param session The session on which communication is happening
  * \param tdata The message to add the header to
  * \param id The identification information used to populate the header
  */
@@ -632,6 +708,7 @@ static void add_rpid_header(const struct ast_sip_session *session, pjsip_tx_data
 			pj_list_erase(old_rpid);
 		} else {
 			ast_sip_modify_id_header(tdata->pool, old_rpid, id);
+			add_party_param(tdata, old_rpid, session);
 			add_privacy_params(tdata, old_rpid, id);
 			return;
 		}
@@ -647,6 +724,7 @@ static void add_rpid_header(const struct ast_sip_session *session, pjsip_tx_data
 	if (!rpid_hdr) {
 		return;
 	}
+	add_party_param(tdata, rpid_hdr, session);
 	add_privacy_params(tdata, rpid_hdr, id);
 	pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr *)rpid_hdr);
 }
@@ -660,7 +738,7 @@ static void add_rpid_header(const struct ast_sip_session *session, pjsip_tx_data
  *
  * \param session The session on which we will be sending the message
  * \param tdata The outbound message
- * \param The identity information to place on the message
+ * \param id The identity information to place on the message
  */
 static void add_id_headers(const struct ast_sip_session *session, pjsip_tx_data *tdata, const struct ast_party_id *id)
 {
@@ -721,7 +799,10 @@ static void caller_id_outgoing_response(struct ast_sip_session *session, pjsip_t
 	struct ast_party_id effective_id;
 	struct ast_party_id connected_id;
 
-	if (!session->channel) {
+	if (!session->channel
+		|| (!session->endpoint->send_connected_line
+			&& session->inv_session
+			&& session->inv_session->state >= PJSIP_INV_STATE_EARLY)) {
 		return;
 	}
 
@@ -747,8 +828,7 @@ static struct ast_sip_session_supplement caller_id_supplement = {
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
+	ast_module_shutdown_ref(AST_MODULE_SELF);
 	ast_sip_session_register_supplement(&caller_id_supplement);
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -764,4 +844,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Caller ID Suppo
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
 );
